# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ....._types import Body, Omit, Query, Headers, NoneType, NotGiven, omit, not_given
from ....._utils import maybe_transform, async_maybe_transform
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource
from ....._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .....pagination import SyncCursor, AsyncCursor
from ....._base_client import AsyncPaginator, make_request_options
from .....types.senders.agent.agent_document import AgentDocument
from .....types.senders.agent.knowledge_bases import document_list_params, document_create_params
from .....types.senders.agent.knowledge_bases.document_create_response import DocumentCreateResponse

__all__ = ["DocumentsResource", "AsyncDocumentsResource"]


class DocumentsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> DocumentsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/zavudev/sdk-python#accessing-raw-response-data-eg-headers
        """
        return DocumentsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> DocumentsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/zavudev/sdk-python#with_streaming_response
        """
        return DocumentsResourceWithStreamingResponse(self)

    def create(
        self,
        kb_id: str,
        *,
        sender_id: str,
        content: str,
        title: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DocumentCreateResponse:
        """Add a document to a knowledge base.

        The document will be automatically processed
        for RAG.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not sender_id:
            raise ValueError(f"Expected a non-empty value for `sender_id` but received {sender_id!r}")
        if not kb_id:
            raise ValueError(f"Expected a non-empty value for `kb_id` but received {kb_id!r}")
        return self._post(
            f"/v1/senders/{sender_id}/agent/knowledge-bases/{kb_id}/documents",
            body=maybe_transform(
                {
                    "content": content,
                    "title": title,
                },
                document_create_params.DocumentCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DocumentCreateResponse,
        )

    def list(
        self,
        kb_id: str,
        *,
        sender_id: str,
        cursor: str | Omit = omit,
        limit: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncCursor[AgentDocument]:
        """
        List documents in a knowledge base.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not sender_id:
            raise ValueError(f"Expected a non-empty value for `sender_id` but received {sender_id!r}")
        if not kb_id:
            raise ValueError(f"Expected a non-empty value for `kb_id` but received {kb_id!r}")
        return self._get_api_list(
            f"/v1/senders/{sender_id}/agent/knowledge-bases/{kb_id}/documents",
            page=SyncCursor[AgentDocument],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "cursor": cursor,
                        "limit": limit,
                    },
                    document_list_params.DocumentListParams,
                ),
            ),
            model=AgentDocument,
        )

    def delete(
        self,
        doc_id: str,
        *,
        sender_id: str,
        kb_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Delete a document from a knowledge base.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not sender_id:
            raise ValueError(f"Expected a non-empty value for `sender_id` but received {sender_id!r}")
        if not kb_id:
            raise ValueError(f"Expected a non-empty value for `kb_id` but received {kb_id!r}")
        if not doc_id:
            raise ValueError(f"Expected a non-empty value for `doc_id` but received {doc_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._delete(
            f"/v1/senders/{sender_id}/agent/knowledge-bases/{kb_id}/documents/{doc_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class AsyncDocumentsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncDocumentsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/zavudev/sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncDocumentsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncDocumentsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/zavudev/sdk-python#with_streaming_response
        """
        return AsyncDocumentsResourceWithStreamingResponse(self)

    async def create(
        self,
        kb_id: str,
        *,
        sender_id: str,
        content: str,
        title: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DocumentCreateResponse:
        """Add a document to a knowledge base.

        The document will be automatically processed
        for RAG.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not sender_id:
            raise ValueError(f"Expected a non-empty value for `sender_id` but received {sender_id!r}")
        if not kb_id:
            raise ValueError(f"Expected a non-empty value for `kb_id` but received {kb_id!r}")
        return await self._post(
            f"/v1/senders/{sender_id}/agent/knowledge-bases/{kb_id}/documents",
            body=await async_maybe_transform(
                {
                    "content": content,
                    "title": title,
                },
                document_create_params.DocumentCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DocumentCreateResponse,
        )

    def list(
        self,
        kb_id: str,
        *,
        sender_id: str,
        cursor: str | Omit = omit,
        limit: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[AgentDocument, AsyncCursor[AgentDocument]]:
        """
        List documents in a knowledge base.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not sender_id:
            raise ValueError(f"Expected a non-empty value for `sender_id` but received {sender_id!r}")
        if not kb_id:
            raise ValueError(f"Expected a non-empty value for `kb_id` but received {kb_id!r}")
        return self._get_api_list(
            f"/v1/senders/{sender_id}/agent/knowledge-bases/{kb_id}/documents",
            page=AsyncCursor[AgentDocument],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "cursor": cursor,
                        "limit": limit,
                    },
                    document_list_params.DocumentListParams,
                ),
            ),
            model=AgentDocument,
        )

    async def delete(
        self,
        doc_id: str,
        *,
        sender_id: str,
        kb_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Delete a document from a knowledge base.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not sender_id:
            raise ValueError(f"Expected a non-empty value for `sender_id` but received {sender_id!r}")
        if not kb_id:
            raise ValueError(f"Expected a non-empty value for `kb_id` but received {kb_id!r}")
        if not doc_id:
            raise ValueError(f"Expected a non-empty value for `doc_id` but received {doc_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._delete(
            f"/v1/senders/{sender_id}/agent/knowledge-bases/{kb_id}/documents/{doc_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class DocumentsResourceWithRawResponse:
    def __init__(self, documents: DocumentsResource) -> None:
        self._documents = documents

        self.create = to_raw_response_wrapper(
            documents.create,
        )
        self.list = to_raw_response_wrapper(
            documents.list,
        )
        self.delete = to_raw_response_wrapper(
            documents.delete,
        )


class AsyncDocumentsResourceWithRawResponse:
    def __init__(self, documents: AsyncDocumentsResource) -> None:
        self._documents = documents

        self.create = async_to_raw_response_wrapper(
            documents.create,
        )
        self.list = async_to_raw_response_wrapper(
            documents.list,
        )
        self.delete = async_to_raw_response_wrapper(
            documents.delete,
        )


class DocumentsResourceWithStreamingResponse:
    def __init__(self, documents: DocumentsResource) -> None:
        self._documents = documents

        self.create = to_streamed_response_wrapper(
            documents.create,
        )
        self.list = to_streamed_response_wrapper(
            documents.list,
        )
        self.delete = to_streamed_response_wrapper(
            documents.delete,
        )


class AsyncDocumentsResourceWithStreamingResponse:
    def __init__(self, documents: AsyncDocumentsResource) -> None:
        self._documents = documents

        self.create = async_to_streamed_response_wrapper(
            documents.create,
        )
        self.list = async_to_streamed_response_wrapper(
            documents.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            documents.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["SenderUploadProfilePictureParams"]


class SenderUploadProfilePictureParams(TypedDict, total=False):
    image_url: Required[Annotated[str, PropertyInfo(alias="imageUrl")]]
    """URL of the image to upload."""

    mime_type: Required[Annotated[Literal["image/jpeg", "image/png"], PropertyInfo(alias="mimeType")]]
    """MIME type of the image."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..types import address_list_params, address_create_params
from .._types import Body, Omit, Query, Headers, NoneType, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncCursor, AsyncCursor
from .._base_client import AsyncPaginator, make_request_options
from ..types.address import Address
from ..types.address_create_response import AddressCreateResponse
from ..types.address_retrieve_response import AddressRetrieveResponse

__all__ = ["AddressesResource", "AsyncAddressesResource"]


class AddressesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AddressesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/zavudev/sdk-python#accessing-raw-response-data-eg-headers
        """
        return AddressesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AddressesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/zavudev/sdk-python#with_streaming_response
        """
        return AddressesResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        country_code: str,
        locality: str,
        postal_code: str,
        street_address: str,
        administrative_area: str | Omit = omit,
        business_name: str | Omit = omit,
        extended_address: str | Omit = omit,
        first_name: str | Omit = omit,
        last_name: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AddressCreateResponse:
        """Create a regulatory address for phone number purchases.

        Some countries require a
        verified address before phone numbers can be activated.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v1/addresses",
            body=maybe_transform(
                {
                    "country_code": country_code,
                    "locality": locality,
                    "postal_code": postal_code,
                    "street_address": street_address,
                    "administrative_area": administrative_area,
                    "business_name": business_name,
                    "extended_address": extended_address,
                    "first_name": first_name,
                    "last_name": last_name,
                },
                address_create_params.AddressCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AddressCreateResponse,
        )

    def retrieve(
        self,
        address_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AddressRetrieveResponse:
        """
        Get a specific regulatory address.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not address_id:
            raise ValueError(f"Expected a non-empty value for `address_id` but received {address_id!r}")
        return self._get(
            f"/v1/addresses/{address_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AddressRetrieveResponse,
        )

    def list(
        self,
        *,
        cursor: str | Omit = omit,
        limit: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncCursor[Address]:
        """
        List regulatory addresses for this project.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v1/addresses",
            page=SyncCursor[Address],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "cursor": cursor,
                        "limit": limit,
                    },
                    address_list_params.AddressListParams,
                ),
            ),
            model=Address,
        )

    def delete(
        self,
        address_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """Delete a regulatory address.

        Cannot delete addresses that are in use.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not address_id:
            raise ValueError(f"Expected a non-empty value for `address_id` but received {address_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._delete(
            f"/v1/addresses/{address_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class AsyncAddressesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncAddressesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/zavudev/sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncAddressesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAddressesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/zavudev/sdk-python#with_streaming_response
        """
        return AsyncAddressesResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        country_code: str,
        locality: str,
        postal_code: str,
        street_address: str,
        administrative_area: str | Omit = omit,
        business_name: str | Omit = omit,
        extended_address: str | Omit = omit,
        first_name: str | Omit = omit,
        last_name: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AddressCreateResponse:
        """Create a regulatory address for phone number purchases.

        Some countries require a
        verified address before phone numbers can be activated.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v1/addresses",
            body=await async_maybe_transform(
                {
                    "country_code": country_code,
                    "locality": locality,
                    "postal_code": postal_code,
                    "street_address": street_address,
                    "administrative_area": administrative_area,
                    "business_name": business_name,
                    "extended_address": extended_address,
                    "first_name": first_name,
                    "last_name": last_name,
                },
                address_create_params.AddressCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AddressCreateResponse,
        )

    async def retrieve(
        self,
        address_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AddressRetrieveResponse:
        """
        Get a specific regulatory address.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not address_id:
            raise ValueError(f"Expected a non-empty value for `address_id` but received {address_id!r}")
        return await self._get(
            f"/v1/addresses/{address_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AddressRetrieveResponse,
        )

    def list(
        self,
        *,
        cursor: str | Omit = omit,
        limit: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[Address, AsyncCursor[Address]]:
        """
        List regulatory addresses for this project.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v1/addresses",
            page=AsyncCursor[Address],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "cursor": cursor,
                        "limit": limit,
                    },
                    address_list_params.AddressListParams,
                ),
            ),
            model=Address,
        )

    async def delete(
        self,
        address_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """Delete a regulatory address.

        Cannot delete addresses that are in use.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not address_id:
            raise ValueError(f"Expected a non-empty value for `address_id` but received {address_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._delete(
            f"/v1/addresses/{address_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class AddressesResourceWithRawResponse:
    def __init__(self, addresses: AddressesResource) -> None:
        self._addresses = addresses

        self.create = to_raw_response_wrapper(
            addresses.create,
        )
        self.retrieve = to_raw_response_wrapper(
            addresses.retrieve,
        )
        self.list = to_raw_response_wrapper(
            addresses.list,
        )
        self.delete = to_raw_response_wrapper(
            addresses.delete,
        )


class AsyncAddressesResourceWithRawResponse:
    def __init__(self, addresses: AsyncAddressesResource) -> None:
        self._addresses = addresses

        self.create = async_to_raw_response_wrapper(
            addresses.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            addresses.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            addresses.list,
        )
        self.delete = async_to_raw_response_wrapper(
            addresses.delete,
        )


class AddressesResourceWithStreamingResponse:
    def __init__(self, addresses: AddressesResource) -> None:
        self._addresses = addresses

        self.create = to_streamed_response_wrapper(
            addresses.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            addresses.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            addresses.list,
        )
        self.delete = to_streamed_response_wrapper(
            addresses.delete,
        )


class AsyncAddressesResourceWithStreamingResponse:
    def __init__(self, addresses: AsyncAddressesResource) -> None:
        self._addresses = addresses

        self.create = async_to_streamed_response_wrapper(
            addresses.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            addresses.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            addresses.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            addresses.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal, TypeAlias

__all__ = ["BroadcastMessageType"]

BroadcastMessageType: TypeAlias = Literal["text", "image", "video", "audio", "document", "template"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import TYPE_CHECKING, Any, Mapping
from typing_extensions import Self, override

import httpx

from . import _exceptions
from ._qs import Querystring
from ._types import (
    Omit,
    Timeout,
    NotGiven,
    Transport,
    ProxiesTypes,
    RequestOptions,
    not_given,
)
from ._utils import is_given, get_async_library
from ._compat import cached_property
from ._version import __version__
from ._streaming import Stream as Stream, AsyncStream as AsyncStream
from ._exceptions import ZavudevError, APIStatusError
from ._base_client import (
    DEFAULT_MAX_RETRIES,
    SyncAPIClient,
    AsyncAPIClient,
)

if TYPE_CHECKING:
    from .resources import (
        senders,
        contacts,
        messages,
        addresses,
        templates,
        broadcasts,
        introspect,
        phone_numbers,
        regulatory_documents,
    )
    from .resources.contacts import ContactsResource, AsyncContactsResource
    from .resources.messages import MessagesResource, AsyncMessagesResource
    from .resources.addresses import AddressesResource, AsyncAddressesResource
    from .resources.templates import TemplatesResource, AsyncTemplatesResource
    from .resources.introspect import IntrospectResource, AsyncIntrospectResource
    from .resources.phone_numbers import PhoneNumbersResource, AsyncPhoneNumbersResource
    from .resources.senders.senders import SendersResource, AsyncSendersResource
    from .resources.regulatory_documents import RegulatoryDocumentsResource, AsyncRegulatoryDocumentsResource
    from .resources.broadcasts.broadcasts import BroadcastsResource, AsyncBroadcastsResource

__all__ = ["Timeout", "Transport", "ProxiesTypes", "RequestOptions", "Zavudev", "AsyncZavudev", "Client", "AsyncClient"]


class Zavudev(SyncAPIClient):
    # client options
    api_key: str

    def __init__(
        self,
        *,
        api_key: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        max_retries: int = DEFAULT_MAX_RETRIES,
        default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        # Configure a custom httpx client.
        # We provide a `DefaultHttpxClient` class that you can pass to retain the default values we use for `limits`, `timeout` & `follow_redirects`.
        # See the [httpx documentation](https://www.python-httpx.org/api/#client) for more details.
        http_client: httpx.Client | None = None,
        # Enable or disable schema validation for data returned by the API.
        # When enabled an error APIResponseValidationError is raised
        # if the API responds with invalid data for the expected schema.
        #
        # This parameter may be removed or changed in the future.
        # If you rely on this feature, please open a GitHub issue
        # outlining your use-case to help us decide if it should be
        # part of our public interface in the future.
        _strict_response_validation: bool = False,
    ) -> None:
        """Construct a new synchronous Zavudev client instance.

        This automatically infers the `api_key` argument from the `ZAVUDEV_API_KEY` environment variable if it is not provided.
        """
        if api_key is None:
            api_key = os.environ.get("ZAVUDEV_API_KEY")
        if api_key is None:
            raise ZavudevError(
                "The api_key client option must be set either by passing api_key to the client or by setting the ZAVUDEV_API_KEY environment variable"
            )
        self.api_key = api_key

        if base_url is None:
            base_url = os.environ.get("ZAVUDEV_BASE_URL")
        if base_url is None:
            base_url = f"https://api.zavu.dev"

        super().__init__(
            version=__version__,
            base_url=base_url,
            max_retries=max_retries,
            timeout=timeout,
            http_client=http_client,
            custom_headers=default_headers,
            custom_query=default_query,
            _strict_response_validation=_strict_response_validation,
        )

    @cached_property
    def messages(self) -> MessagesResource:
        from .resources.messages import MessagesResource

        return MessagesResource(self)

    @cached_property
    def templates(self) -> TemplatesResource:
        from .resources.templates import TemplatesResource

        return TemplatesResource(self)

    @cached_property
    def senders(self) -> SendersResource:
        from .resources.senders import SendersResource

        return SendersResource(self)

    @cached_property
    def contacts(self) -> ContactsResource:
        from .resources.contacts import ContactsResource

        return ContactsResource(self)

    @cached_property
    def broadcasts(self) -> BroadcastsResource:
        from .resources.broadcasts import BroadcastsResource

        return BroadcastsResource(self)

    @cached_property
    def introspect(self) -> IntrospectResource:
        from .resources.introspect import IntrospectResource

        return IntrospectResource(self)

    @cached_property
    def phone_numbers(self) -> PhoneNumbersResource:
        from .resources.phone_numbers import PhoneNumbersResource

        return PhoneNumbersResource(self)

    @cached_property
    def addresses(self) -> AddressesResource:
        from .resources.addresses import AddressesResource

        return AddressesResource(self)

    @cached_property
    def regulatory_documents(self) -> RegulatoryDocumentsResource:
        from .resources.regulatory_documents import RegulatoryDocumentsResource

        return RegulatoryDocumentsResource(self)

    @cached_property
    def with_raw_response(self) -> ZavudevWithRawResponse:
        return ZavudevWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ZavudevWithStreamedResponse:
        return ZavudevWithStreamedResponse(self)

    @property
    @override
    def qs(self) -> Querystring:
        return Querystring(array_format="comma")

    @property
    @override
    def auth_headers(self) -> dict[str, str]:
        api_key = self.api_key
        return {"Authorization": f"Bearer {api_key}"}

    @property
    @override
    def default_headers(self) -> dict[str, str | Omit]:
        return {
            **super().default_headers,
            "X-Stainless-Async": "false",
            **self._custom_headers,
        }

    def copy(
        self,
        *,
        api_key: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        http_client: httpx.Client | None = None,
        max_retries: int | NotGiven = not_given,
        default_headers: Mapping[str, str] | None = None,
        set_default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        set_default_query: Mapping[str, object] | None = None,
        _extra_kwargs: Mapping[str, Any] = {},
    ) -> Self:
        """
        Create a new client instance re-using the same options given to the current client with optional overriding.
        """
        if default_headers is not None and set_default_headers is not None:
            raise ValueError("The `default_headers` and `set_default_headers` arguments are mutually exclusive")

        if default_query is not None and set_default_query is not None:
            raise ValueError("The `default_query` and `set_default_query` arguments are mutually exclusive")

        headers = self._custom_headers
        if default_headers is not None:
            headers = {**headers, **default_headers}
        elif set_default_headers is not None:
            headers = set_default_headers

        params = self._custom_query
        if default_query is not None:
            params = {**params, **default_query}
        elif set_default_query is not None:
            params = set_default_query

        http_client = http_client or self._client
        return self.__class__(
            api_key=api_key or self.api_key,
            base_url=base_url or self.base_url,
            timeout=self.timeout if isinstance(timeout, NotGiven) else timeout,
            http_client=http_client,
            max_retries=max_retries if is_given(max_retries) else self.max_retries,
            default_headers=headers,
            default_query=params,
            **_extra_kwargs,
        )

    # Alias for `copy` for nicer inline usage, e.g.
    # client.with_options(timeout=10).foo.create(...)
    with_options = copy

    @override
    def _make_status_error(
        self,
        err_msg: str,
        *,
        body: object,
        response: httpx.Response,
    ) -> APIStatusError:
        if response.status_code == 400:
            return _exceptions.BadRequestError(err_msg, response=response, body=body)

        if response.status_code == 401:
            return _exceptions.AuthenticationError(err_msg, response=response, body=body)

        if response.status_code == 403:
            return _exceptions.PermissionDeniedError(err_msg, response=response, body=body)

        if response.status_code == 404:
            return _exceptions.NotFoundError(err_msg, response=response, body=body)

        if response.status_code == 409:
            return _exceptions.ConflictError(err_msg, response=response, body=body)

        if response.status_code == 422:
            return _exceptions.UnprocessableEntityError(err_msg, response=response, body=body)

        if response.status_code == 429:
            return _exceptions.RateLimitError(err_msg, response=response, body=body)

        if response.status_code >= 500:
            return _exceptions.InternalServerError(err_msg, response=response, body=body)
        return APIStatusError(err_msg, response=response, body=body)


class AsyncZavudev(AsyncAPIClient):
    # client options
    api_key: str

    def __init__(
        self,
        *,
        api_key: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        max_retries: int = DEFAULT_MAX_RETRIES,
        default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        # Configure a custom httpx client.
        # We provide a `DefaultAsyncHttpxClient` class that you can pass to retain the default values we use for `limits`, `timeout` & `follow_redirects`.
        # See the [httpx documentation](https://www.python-httpx.org/api/#asyncclient) for more details.
        http_client: httpx.AsyncClient | None = None,
        # Enable or disable schema validation for data returned by the API.
        # When enabled an error APIResponseValidationError is raised
        # if the API responds with invalid data for the expected schema.
        #
        # This parameter may be removed or changed in the future.
        # If you rely on this feature, please open a GitHub issue
        # outlining your use-case to help us decide if it should be
        # part of our public interface in the future.
        _strict_response_validation: bool = False,
    ) -> None:
        """Construct a new async AsyncZavudev client instance.

        This automatically infers the `api_key` argument from the `ZAVUDEV_API_KEY` environment variable if it is not provided.
        """
        if api_key is None:
            api_key = os.environ.get("ZAVUDEV_API_KEY")
        if api_key is None:
            raise ZavudevError(
                "The api_key client option must be set either by passing api_key to the client or by setting the ZAVUDEV_API_KEY environment variable"
            )
        self.api_key = api_key

        if base_url is None:
            base_url = os.environ.get("ZAVUDEV_BASE_URL")
        if base_url is None:
            base_url = f"https://api.zavu.dev"

        super().__init__(
            version=__version__,
            base_url=base_url,
            max_retries=max_retries,
            timeout=timeout,
            http_client=http_client,
            custom_headers=default_headers,
            custom_query=default_query,
            _strict_response_validation=_strict_response_validation,
        )

    @cached_property
    def messages(self) -> AsyncMessagesResource:
        from .resources.messages import AsyncMessagesResource

        return AsyncMessagesResource(self)

    @cached_property
    def templates(self) -> AsyncTemplatesResource:
        from .resources.templates import AsyncTemplatesResource

        return AsyncTemplatesResource(self)

    @cached_property
    def senders(self) -> AsyncSendersResource:
        from .resources.senders import AsyncSendersResource

        return AsyncSendersResource(self)

    @cached_property
    def contacts(self) -> AsyncContactsResource:
        from .resources.contacts import AsyncContactsResource

        return AsyncContactsResource(self)

    @cached_property
    def broadcasts(self) -> AsyncBroadcastsResource:
        from .resources.broadcasts import AsyncBroadcastsResource

        return AsyncBroadcastsResource(self)

    @cached_property
    def introspect(self) -> AsyncIntrospectResource:
        from .resources.introspect import AsyncIntrospectResource

        return AsyncIntrospectResource(self)

    @cached_property
    def phone_numbers(self) -> AsyncPhoneNumbersResource:
        from .resources.phone_numbers import AsyncPhoneNumbersResource

        return AsyncPhoneNumbersResource(self)

    @cached_property
    def addresses(self) -> AsyncAddressesResource:
        from .resources.addresses import AsyncAddressesResource

        return AsyncAddressesResource(self)

    @cached_property
    def regulatory_documents(self) -> AsyncRegulatoryDocumentsResource:
        from .resources.regulatory_documents import AsyncRegulatoryDocumentsResource

        return AsyncRegulatoryDocumentsResource(self)

    @cached_property
    def with_raw_response(self) -> AsyncZavudevWithRawResponse:
        return AsyncZavudevWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncZavudevWithStreamedResponse:
        return AsyncZavudevWithStreamedResponse(self)

    @property
    @override
    def qs(self) -> Querystring:
        return Querystring(array_format="comma")

    @property
    @override
    def auth_headers(self) -> dict[str, str]:
        api_key = self.api_key
        return {"Authorization": f"Bearer {api_key}"}

    @property
    @override
    def default_headers(self) -> dict[str, str | Omit]:
        return {
            **super().default_headers,
            "X-Stainless-Async": f"async:{get_async_library()}",
            **self._custom_headers,
        }

    def copy(
        self,
        *,
        api_key: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        http_client: httpx.AsyncClient | None = None,
        max_retries: int | NotGiven = not_given,
        default_headers: Mapping[str, str] | None = None,
        set_default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        set_default_query: Mapping[str, object] | None = None,
        _extra_kwargs: Mapping[str, Any] = {},
    ) -> Self:
        """
        Create a new client instance re-using the same options given to the current client with optional overriding.
        """
        if default_headers is not None and set_default_headers is not None:
            raise ValueError("The `default_headers` and `set_default_headers` arguments are mutually exclusive")

        if default_query is not None and set_default_query is not None:
            raise ValueError("The `default_query` and `set_default_query` arguments are mutually exclusive")

        headers = self._custom_headers
        if default_headers is not None:
            headers = {**headers, **default_headers}
        elif set_default_headers is not None:
            headers = set_default_headers

        params = self._custom_query
        if default_query is not None:
            params = {**params, **default_query}
        elif set_default_query is not None:
            params = set_default_query

        http_client = http_client or self._client
        return self.__class__(
            api_key=api_key or self.api_key,
            base_url=base_url or self.base_url,
            timeout=self.timeout if isinstance(timeout, NotGiven) else timeout,
            http_client=http_client,
            max_retries=max_retries if is_given(max_retries) else self.max_retries,
            default_headers=headers,
            default_query=params,
            **_extra_kwargs,
        )

    # Alias for `copy` for nicer inline usage, e.g.
    # client.with_options(timeout=10).foo.create(...)
    with_options = copy

    @override
    def _make_status_error(
        self,
        err_msg: str,
        *,
        body: object,
        response: httpx.Response,
    ) -> APIStatusError:
        if response.status_code == 400:
            return _exceptions.BadRequestError(err_msg, response=response, body=body)

        if response.status_code == 401:
            return _exceptions.AuthenticationError(err_msg, response=response, body=body)

        if response.status_code == 403:
            return _exceptions.PermissionDeniedError(err_msg, response=response, body=body)

        if response.status_code == 404:
            return _exceptions.NotFoundError(err_msg, response=response, body=body)

        if response.status_code == 409:
            return _exceptions.ConflictError(err_msg, response=response, body=body)

        if response.status_code == 422:
            return _exceptions.UnprocessableEntityError(err_msg, response=response, body=body)

        if response.status_code == 429:
            return _exceptions.RateLimitError(err_msg, response=response, body=body)

        if response.status_code >= 500:
            return _exceptions.InternalServerError(err_msg, response=response, body=body)
        return APIStatusError(err_msg, response=response, body=body)


class ZavudevWithRawResponse:
    _client: Zavudev

    def __init__(self, client: Zavudev) -> None:
        self._client = client

    @cached_property
    def messages(self) -> messages.MessagesResourceWithRawResponse:
        from .resources.messages import MessagesResourceWithRawResponse

        return MessagesResourceWithRawResponse(self._client.messages)

    @cached_property
    def templates(self) -> templates.TemplatesResourceWithRawResponse:
        from .resources.templates import TemplatesResourceWithRawResponse

        return TemplatesResourceWithRawResponse(self._client.templates)

    @cached_property
    def senders(self) -> senders.SendersResourceWithRawResponse:
        from .resources.senders import SendersResourceWithRawResponse

        return SendersResourceWithRawResponse(self._client.senders)

    @cached_property
    def contacts(self) -> contacts.ContactsResourceWithRawResponse:
        from .resources.contacts import ContactsResourceWithRawResponse

        return ContactsResourceWithRawResponse(self._client.contacts)

    @cached_property
    def broadcasts(self) -> broadcasts.BroadcastsResourceWithRawResponse:
        from .resources.broadcasts import BroadcastsResourceWithRawResponse

        return BroadcastsResourceWithRawResponse(self._client.broadcasts)

    @cached_property
    def introspect(self) -> introspect.IntrospectResourceWithRawResponse:
        from .resources.introspect import IntrospectResourceWithRawResponse

        return IntrospectResourceWithRawResponse(self._client.introspect)

    @cached_property
    def phone_numbers(self) -> phone_numbers.PhoneNumbersResourceWithRawResponse:
        from .resources.phone_numbers import PhoneNumbersResourceWithRawResponse

        return PhoneNumbersResourceWithRawResponse(self._client.phone_numbers)

    @cached_property
    def addresses(self) -> addresses.AddressesResourceWithRawResponse:
        from .resources.addresses import AddressesResourceWithRawResponse

        return AddressesResourceWithRawResponse(self._client.addresses)

    @cached_property
    def regulatory_documents(self) -> regulatory_documents.RegulatoryDocumentsResourceWithRawResponse:
        from .resources.regulatory_documents import RegulatoryDocumentsResourceWithRawResponse

        return RegulatoryDocumentsResourceWithRawResponse(self._client.regulatory_documents)


class AsyncZavudevWithRawResponse:
    _client: AsyncZavudev

    def __init__(self, client: AsyncZavudev) -> None:
        self._client = client

    @cached_property
    def messages(self) -> messages.AsyncMessagesResourceWithRawResponse:
        from .resources.messages import AsyncMessagesResourceWithRawResponse

        return AsyncMessagesResourceWithRawResponse(self._client.messages)

    @cached_property
    def templates(self) -> templates.AsyncTemplatesResourceWithRawResponse:
        from .resources.templates import AsyncTemplatesResourceWithRawResponse

        return AsyncTemplatesResourceWithRawResponse(self._client.templates)

    @cached_property
    def senders(self) -> senders.AsyncSendersResourceWithRawResponse:
        from .resources.senders import AsyncSendersResourceWithRawResponse

        return AsyncSendersResourceWithRawResponse(self._client.senders)

    @cached_property
    def contacts(self) -> contacts.AsyncContactsResourceWithRawResponse:
        from .resources.contacts import AsyncContactsResourceWithRawResponse

        return AsyncContactsResourceWithRawResponse(self._client.contacts)

    @cached_property
    def broadcasts(self) -> broadcasts.AsyncBroadcastsResourceWithRawResponse:
        from .resources.broadcasts import AsyncBroadcastsResourceWithRawResponse

        return AsyncBroadcastsResourceWithRawResponse(self._client.broadcasts)

    @cached_property
    def introspect(self) -> introspect.AsyncIntrospectResourceWithRawResponse:
        from .resources.introspect import AsyncIntrospectResourceWithRawResponse

        return AsyncIntrospectResourceWithRawResponse(self._client.introspect)

    @cached_property
    def phone_numbers(self) -> phone_numbers.AsyncPhoneNumbersResourceWithRawResponse:
        from .resources.phone_numbers import AsyncPhoneNumbersResourceWithRawResponse

        return AsyncPhoneNumbersResourceWithRawResponse(self._client.phone_numbers)

    @cached_property
    def addresses(self) -> addresses.AsyncAddressesResourceWithRawResponse:
        from .resources.addresses import AsyncAddressesResourceWithRawResponse

        return AsyncAddressesResourceWithRawResponse(self._client.addresses)

    @cached_property
    def regulatory_documents(self) -> regulatory_documents.AsyncRegulatoryDocumentsResourceWithRawResponse:
        from .resources.regulatory_documents import AsyncRegulatoryDocumentsResourceWithRawResponse

        return AsyncRegulatoryDocumentsResourceWithRawResponse(self._client.regulatory_documents)


class ZavudevWithStreamedResponse:
    _client: Zavudev

    def __init__(self, client: Zavudev) -> None:
        self._client = client

    @cached_property
    def messages(self) -> messages.MessagesResourceWithStreamingResponse:
        from .resources.messages import MessagesResourceWithStreamingResponse

        return MessagesResourceWithStreamingResponse(self._client.messages)

    @cached_property
    def templates(self) -> templates.TemplatesResourceWithStreamingResponse:
        from .resources.templates import TemplatesResourceWithStreamingResponse

        return TemplatesResourceWithStreamingResponse(self._client.templates)

    @cached_property
    def senders(self) -> senders.SendersResourceWithStreamingResponse:
        from .resources.senders import SendersResourceWithStreamingResponse

        return SendersResourceWithStreamingResponse(self._client.senders)

    @cached_property
    def contacts(self) -> contacts.ContactsResourceWithStreamingResponse:
        from .resources.contacts import ContactsResourceWithStreamingResponse

        return ContactsResourceWithStreamingResponse(self._client.contacts)

    @cached_property
    def broadcasts(self) -> broadcasts.BroadcastsResourceWithStreamingResponse:
        from .resources.broadcasts import BroadcastsResourceWithStreamingResponse

        return BroadcastsResourceWithStreamingResponse(self._client.broadcasts)

    @cached_property
    def introspect(self) -> introspect.IntrospectResourceWithStreamingResponse:
        from .resources.introspect import IntrospectResourceWithStreamingResponse

        return IntrospectResourceWithStreamingResponse(self._client.introspect)

    @cached_property
    def phone_numbers(self) -> phone_numbers.PhoneNumbersResourceWithStreamingResponse:
        from .resources.phone_numbers import PhoneNumbersResourceWithStreamingResponse

        return PhoneNumbersResourceWithStreamingResponse(self._client.phone_numbers)

    @cached_property
    def addresses(self) -> addresses.AddressesResourceWithStreamingResponse:
        from .resources.addresses import AddressesResourceWithStreamingResponse

        return AddressesResourceWithStreamingResponse(self._client.addresses)

    @cached_property
    def regulatory_documents(self) -> regulatory_documents.RegulatoryDocumentsResourceWithStreamingResponse:
        from .resources.regulatory_documents import RegulatoryDocumentsResourceWithStreamingResponse

        return RegulatoryDocumentsResourceWithStreamingResponse(self._client.regulatory_documents)


class AsyncZavudevWithStreamedResponse:
    _client: AsyncZavudev

    def __init__(self, client: AsyncZavudev) -> None:
        self._client = client

    @cached_property
    def messages(self) -> messages.AsyncMessagesResourceWithStreamingResponse:
        from .resources.messages import AsyncMessagesResourceWithStreamingResponse

        return AsyncMessagesResourceWithStreamingResponse(self._client.messages)

    @cached_property
    def templates(self) -> templates.AsyncTemplatesResourceWithStreamingResponse:
        from .resources.templates import AsyncTemplatesResourceWithStreamingResponse

        return AsyncTemplatesResourceWithStreamingResponse(self._client.templates)

    @cached_property
    def senders(self) -> senders.AsyncSendersResourceWithStreamingResponse:
        from .resources.senders import AsyncSendersResourceWithStreamingResponse

        return AsyncSendersResourceWithStreamingResponse(self._client.senders)

    @cached_property
    def contacts(self) -> contacts.AsyncContactsResourceWithStreamingResponse:
        from .resources.contacts import AsyncContactsResourceWithStreamingResponse

        return AsyncContactsResourceWithStreamingResponse(self._client.contacts)

    @cached_property
    def broadcasts(self) -> broadcasts.AsyncBroadcastsResourceWithStreamingResponse:
        from .resources.broadcasts import AsyncBroadcastsResourceWithStreamingResponse

        return AsyncBroadcastsResourceWithStreamingResponse(self._client.broadcasts)

    @cached_property
    def introspect(self) -> introspect.AsyncIntrospectResourceWithStreamingResponse:
        from .resources.introspect import AsyncIntrospectResourceWithStreamingResponse

        return AsyncIntrospectResourceWithStreamingResponse(self._client.introspect)

    @cached_property
    def phone_numbers(self) -> phone_numbers.AsyncPhoneNumbersResourceWithStreamingResponse:
        from .resources.phone_numbers import AsyncPhoneNumbersResourceWithStreamingResponse

        return AsyncPhoneNumbersResourceWithStreamingResponse(self._client.phone_numbers)

    @cached_property
    def addresses(self) -> addresses.AsyncAddressesResourceWithStreamingResponse:
        from .resources.addresses import AsyncAddressesResourceWithStreamingResponse

        return AsyncAddressesResourceWithStreamingResponse(self._client.addresses)

    @cached_property
    def regulatory_documents(self) -> regulatory_documents.AsyncRegulatoryDocumentsResourceWithStreamingResponse:
        from .resources.regulatory_documents import AsyncRegulatoryDocumentsResourceWithStreamingResponse

        return AsyncRegulatoryDocumentsResourceWithStreamingResponse(self._client.regulatory_documents)


Client = Zavudev

AsyncClient = AsyncZavudev



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Optional
from datetime import datetime
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from ...._models import BaseModel

__all__ = ["AgentFlow", "Step", "Trigger"]


class Step(BaseModel):
    id: str
    """Unique step identifier."""

    config: Dict[str, object]
    """Step configuration (varies by type)."""

    type: Literal["message", "collect", "condition", "tool", "llm", "transfer"]
    """Type of flow step."""

    next_step_id: Optional[str] = FieldInfo(alias="nextStepId", default=None)
    """ID of the next step to execute."""


class Trigger(BaseModel):
    type: Literal["keyword", "intent", "always", "manual"]
    """Type of trigger for a flow."""

    intent: Optional[str] = None
    """Intent that triggers the flow (for intent type)."""

    keywords: Optional[List[str]] = None
    """Keywords that trigger the flow (for keyword type)."""


class AgentFlow(BaseModel):
    id: str

    agent_id: str = FieldInfo(alias="agentId")

    created_at: datetime = FieldInfo(alias="createdAt")

    enabled: bool

    name: str

    priority: int
    """Priority when multiple flows match (higher = more priority)."""

    steps: List[Step]

    trigger: Trigger

    updated_at: datetime = FieldInfo(alias="updatedAt")

    description: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .senders import (
    SendersResource,
    AsyncSendersResource,
    SendersResourceWithRawResponse,
    AsyncSendersResourceWithRawResponse,
    SendersResourceWithStreamingResponse,
    AsyncSendersResourceWithStreamingResponse,
)
from .contacts import (
    ContactsResource,
    AsyncContactsResource,
    ContactsResourceWithRawResponse,
    AsyncContactsResourceWithRawResponse,
    ContactsResourceWithStreamingResponse,
    AsyncContactsResourceWithStreamingResponse,
)
from .messages import (
    MessagesResource,
    AsyncMessagesResource,
    MessagesResourceWithRawResponse,
    AsyncMessagesResourceWithRawResponse,
    MessagesResourceWithStreamingResponse,
    AsyncMessagesResourceWithStreamingResponse,
)
from .addresses import (
    AddressesResource,
    AsyncAddressesResource,
    AddressesResourceWithRawResponse,
    AsyncAddressesResourceWithRawResponse,
    AddressesResourceWithStreamingResponse,
    AsyncAddressesResourceWithStreamingResponse,
)
from .templates import (
    TemplatesResource,
    AsyncTemplatesResource,
    TemplatesResourceWithRawResponse,
    AsyncTemplatesResourceWithRawResponse,
    TemplatesResourceWithStreamingResponse,
    AsyncTemplatesResourceWithStreamingResponse,
)
from .broadcasts import (
    BroadcastsResource,
    AsyncBroadcastsResource,
    BroadcastsResourceWithRawResponse,
    AsyncBroadcastsResourceWithRawResponse,
    BroadcastsResourceWithStreamingResponse,
    AsyncBroadcastsResourceWithStreamingResponse,
)
from .introspect import (
    IntrospectResource,
    AsyncIntrospectResource,
    IntrospectResourceWithRawResponse,
    AsyncIntrospectResourceWithRawResponse,
    IntrospectResourceWithStreamingResponse,
    AsyncIntrospectResourceWithStreamingResponse,
)
from .phone_numbers import (
    PhoneNumbersResource,
    AsyncPhoneNumbersResource,
    PhoneNumbersResourceWithRawResponse,
    AsyncPhoneNumbersResourceWithRawResponse,
    PhoneNumbersResourceWithStreamingResponse,
    AsyncPhoneNumbersResourceWithStreamingResponse,
)
from .regulatory_documents import (
    RegulatoryDocumentsResource,
    AsyncRegulatoryDocumentsResource,
    RegulatoryDocumentsResourceWithRawResponse,
    AsyncRegulatoryDocumentsResourceWithRawResponse,
    RegulatoryDocumentsResourceWithStreamingResponse,
    AsyncRegulatoryDocumentsResourceWithStreamingResponse,
)

__all__ = [
    "MessagesResource",
    "AsyncMessagesResource",
    "MessagesResourceWithRawResponse",
    "AsyncMessagesResourceWithRawResponse",
    "MessagesResourceWithStreamingResponse",
    "AsyncMessagesResourceWithStreamingResponse",
    "TemplatesResource",
    "AsyncTemplatesResource",
    "TemplatesResourceWithRawResponse",
    "AsyncTemplatesResourceWithRawResponse",
    "TemplatesResourceWithStreamingResponse",
    "AsyncTemplatesResourceWithStreamingResponse",
    "SendersResource",
    "AsyncSendersResource",
    "SendersResourceWithRawResponse",
    "AsyncSendersResourceWithRawResponse",
    "SendersResourceWithStreamingResponse",
    "AsyncSendersResourceWithStreamingResponse",
    "ContactsResource",
    "AsyncContactsResource",
    "ContactsResourceWithRawResponse",
    "AsyncContactsResourceWithRawResponse",
    "ContactsResourceWithStreamingResponse",
    "AsyncContactsResourceWithStreamingResponse",
    "BroadcastsResource",
    "AsyncBroadcastsResource",
    "BroadcastsResourceWithRawResponse",
    "AsyncBroadcastsResourceWithRawResponse",
    "BroadcastsResourceWithStreamingResponse",
    "AsyncBroadcastsResourceWithStreamingResponse",
    "IntrospectResource",
    "AsyncIntrospectResource",
    "IntrospectResourceWithRawResponse",
    "AsyncIntrospectResourceWithRawResponse",
    "IntrospectResourceWithStreamingResponse",
    "AsyncIntrospectResourceWithStreamingResponse",
    "PhoneNumbersResource",
    "AsyncPhoneNumbersResource",
    "PhoneNumbersResourceWithRawResponse",
    "AsyncPhoneNumbersResourceWithRawResponse",
    "PhoneNumbersResourceWithStreamingResponse",
    "AsyncPhoneNumbersResourceWithStreamingResponse",
    "AddressesResource",
    "AsyncAddressesResource",
    "AddressesResourceWithRawResponse",
    "AsyncAddressesResourceWithRawResponse",
    "AddressesResourceWithStreamingResponse",
    "AsyncAddressesResourceWithStreamingResponse",
    "RegulatoryDocumentsResource",
    "AsyncRegulatoryDocumentsResource",
    "RegulatoryDocumentsResourceWithRawResponse",
    "AsyncRegulatoryDocumentsResourceWithRawResponse",
    "RegulatoryDocumentsResourceWithStreamingResponse",
    "AsyncRegulatoryDocumentsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["PhoneNumberUpdateParams"]


class PhoneNumberUpdateParams(TypedDict, total=False):
    name: Optional[str]
    """Custom name for the phone number. Set to null to clear."""

    sender_id: Annotated[Optional[str], PropertyInfo(alias="senderId")]
    """Sender ID to assign the phone number to. Set to null to unassign."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal, TypeAlias

__all__ = ["AgentProvider"]

AgentProvider: TypeAlias = Literal["openai", "anthropic", "google", "mistral", "zavu"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from .._models import BaseModel
from .whatsapp_category import WhatsappCategory

__all__ = ["Template", "Button", "Whatsapp"]


class Button(BaseModel):
    otp_type: Optional[Literal["COPY_CODE", "ONE_TAP"]] = FieldInfo(alias="otpType", default=None)
    """OTP button type. Required when type is 'otp'."""

    package_name: Optional[str] = FieldInfo(alias="packageName", default=None)
    """Android package name. Required for ONE_TAP buttons."""

    phone_number: Optional[str] = FieldInfo(alias="phoneNumber", default=None)

    signature_hash: Optional[str] = FieldInfo(alias="signatureHash", default=None)
    """Android app signature hash. Required for ONE_TAP buttons."""

    text: Optional[str] = None

    type: Optional[Literal["quick_reply", "url", "phone", "otp"]] = None

    url: Optional[str] = None


class Whatsapp(BaseModel):
    """WhatsApp-specific template information."""

    namespace: Optional[str] = None
    """WhatsApp Business Account namespace."""

    status: Optional[str] = None
    """WhatsApp approval status."""

    template_name: Optional[str] = FieldInfo(alias="templateName", default=None)
    """WhatsApp template name."""


class Template(BaseModel):
    id: str

    body: str
    """Template body with variables: {{1}}, {{2}}, etc."""

    category: WhatsappCategory
    """WhatsApp template category."""

    language: str
    """Language code."""

    name: str
    """Template name (must match WhatsApp template name)."""

    add_security_recommendation: Optional[bool] = FieldInfo(alias="addSecurityRecommendation", default=None)
    """Add 'Do not share this code' disclaimer. Only for AUTHENTICATION templates."""

    buttons: Optional[List[Button]] = None
    """Template buttons."""

    code_expiration_minutes: Optional[int] = FieldInfo(alias="codeExpirationMinutes", default=None)
    """Code expiration time in minutes. Only for AUTHENTICATION templates."""

    created_at: Optional[datetime] = FieldInfo(alias="createdAt", default=None)

    footer: Optional[str] = None
    """Footer text for the template."""

    header_content: Optional[str] = FieldInfo(alias="headerContent", default=None)
    """Header content (text or media URL)."""

    header_type: Optional[str] = FieldInfo(alias="headerType", default=None)
    """Type of header (text, image, video, document)."""

    status: Optional[Literal["draft", "pending", "approved", "rejected"]] = None

    updated_at: Optional[datetime] = FieldInfo(alias="updatedAt", default=None)

    variables: Optional[List[str]] = None
    """List of variable names for documentation."""

    whatsapp: Optional[Whatsapp] = None
    """WhatsApp-specific template information."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ....._models import BaseModel
from ..agent_document import AgentDocument

__all__ = ["DocumentCreateResponse"]


class DocumentCreateResponse(BaseModel):
    document: AgentDocument



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .agent import (
    AgentResource,
    AsyncAgentResource,
    AgentResourceWithRawResponse,
    AsyncAgentResourceWithRawResponse,
    AgentResourceWithStreamingResponse,
    AsyncAgentResourceWithStreamingResponse,
)
from .senders import (
    SendersResource,
    AsyncSendersResource,
    SendersResourceWithRawResponse,
    AsyncSendersResourceWithRawResponse,
    SendersResourceWithStreamingResponse,
    AsyncSendersResourceWithStreamingResponse,
)

__all__ = [
    "AgentResource",
    "AsyncAgentResource",
    "AgentResourceWithRawResponse",
    "AsyncAgentResourceWithRawResponse",
    "AgentResourceWithStreamingResponse",
    "AsyncAgentResourceWithStreamingResponse",
    "SendersResource",
    "AsyncSendersResource",
    "SendersResourceWithRawResponse",
    "AsyncSendersResourceWithRawResponse",
    "SendersResourceWithStreamingResponse",
    "AsyncSendersResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from zavudev import Zavudev, AsyncZavudev
from tests.utils import assert_matches_type
from zavudev.types import (
    Sender,
    WebhookSecretResponse,
    SenderUpdateProfileResponse,
    WhatsappBusinessProfileResponse,
    SenderUploadProfilePictureResponse,
)
from zavudev.pagination import SyncCursor, AsyncCursor

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestSenders:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create(self, client: Zavudev) -> None:
        sender = client.senders.create(
            name="name",
            phone_number="phoneNumber",
        )
        assert_matches_type(Sender, sender, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create_with_all_params(self, client: Zavudev) -> None:
        sender = client.senders.create(
            name="name",
            phone_number="phoneNumber",
            set_as_default=True,
            webhook_events=["message.queued"],
            webhook_url="https://example.com",
        )
        assert_matches_type(Sender, sender, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Zavudev) -> None:
        response = client.senders.with_raw_response.create(
            name="name",
            phone_number="phoneNumber",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        sender = response.parse()
        assert_matches_type(Sender, sender, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Zavudev) -> None:
        with client.senders.with_streaming_response.create(
            name="name",
            phone_number="phoneNumber",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            sender = response.parse()
            assert_matches_type(Sender, sender, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Zavudev) -> None:
        sender = client.senders.retrieve(
            "senderId",
        )
        assert_matches_type(Sender, sender, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Zavudev) -> None:
        response = client.senders.with_raw_response.retrieve(
            "senderId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        sender = response.parse()
        assert_matches_type(Sender, sender, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Zavudev) -> None:
        with client.senders.with_streaming_response.retrieve(
            "senderId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            sender = response.parse()
            assert_matches_type(Sender, sender, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Zavudev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `sender_id` but received ''"):
            client.senders.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update(self, client: Zavudev) -> None:
        sender = client.senders.update(
            sender_id="senderId",
        )
        assert_matches_type(Sender, sender, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update_with_all_params(self, client: Zavudev) -> None:
        sender = client.senders.update(
            sender_id="senderId",
            email_receiving_enabled=True,
            name="name",
            set_as_default=True,
            webhook_active=True,
            webhook_events=["message.queued"],
            webhook_url="https://example.com",
        )
        assert_matches_type(Sender, sender, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_update(self, client: Zavudev) -> None:
        response = client.senders.with_raw_response.update(
            sender_id="senderId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        sender = response.parse()
        assert_matches_type(Sender, sender, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_update(self, client: Zavudev) -> None:
        with client.senders.with_streaming_response.update(
            sender_id="senderId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            sender = response.parse()
            assert_matches_type(Sender, sender, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_update(self, client: Zavudev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `sender_id` but received ''"):
            client.senders.with_raw_response.update(
                sender_id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list(self, client: Zavudev) -> None:
        sender = client.senders.list()
        assert_matches_type(SyncCursor[Sender], sender, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Zavudev) -> None:
        sender = client.senders.list(
            cursor="cursor",
            limit=100,
        )
        assert_matches_type(SyncCursor[Sender], sender, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Zavudev) -> None:
        response = client.senders.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        sender = response.parse()
        assert_matches_type(SyncCursor[Sender], sender, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Zavudev) -> None:
        with client.senders.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            sender = response.parse()
            assert_matches_type(SyncCursor[Sender], sender, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_delete(self, client: Zavudev) -> None:
        sender = client.senders.delete(
            "senderId",
        )
        assert sender is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_delete(self, client: Zavudev) -> None:
        response = client.senders.with_raw_response.delete(
            "senderId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        sender = response.parse()
        assert sender is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_delete(self, client: Zavudev) -> None:
        with client.senders.with_streaming_response.delete(
            "senderId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            sender = response.parse()
            assert sender is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_delete(self, client: Zavudev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `sender_id` but received ''"):
            client.senders.with_raw_response.delete(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_get_profile(self, client: Zavudev) -> None:
        sender = client.senders.get_profile(
            "senderId",
        )
        assert_matches_type(WhatsappBusinessProfileResponse, sender, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_get_profile(self, client: Zavudev) -> None:
        response = client.senders.with_raw_response.get_profile(
            "senderId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        sender = response.parse()
        assert_matches_type(WhatsappBusinessProfileResponse, sender, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_get_profile(self, client: Zavudev) -> None:
        with client.senders.with_streaming_response.get_profile(
            "senderId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            sender = response.parse()
            assert_matches_type(WhatsappBusinessProfileResponse, sender, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_get_profile(self, client: Zavudev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `sender_id` but received ''"):
            client.senders.with_raw_response.get_profile(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_regenerate_webhook_secret(self, client: Zavudev) -> None:
        sender = client.senders.regenerate_webhook_secret(
            "senderId",
        )
        assert_matches_type(WebhookSecretResponse, sender, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_regenerate_webhook_secret(self, client: Zavudev) -> None:
        response = client.senders.with_raw_response.regenerate_webhook_secret(
            "senderId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        sender = response.parse()
        assert_matches_type(WebhookSecretResponse, sender, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_regenerate_webhook_secret(self, client: Zavudev) -> None:
        with client.senders.with_streaming_response.regenerate_webhook_secret(
            "senderId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            sender = response.parse()
            assert_matches_type(WebhookSecretResponse, sender, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_regenerate_webhook_secret(self, client: Zavudev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `sender_id` but received ''"):
            client.senders.with_raw_response.regenerate_webhook_secret(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update_profile(self, client: Zavudev) -> None:
        sender = client.senders.update_profile(
            sender_id="senderId",
        )
        assert_matches_type(SenderUpdateProfileResponse, sender, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update_profile_with_all_params(self, client: Zavudev) -> None:
        sender = client.senders.update_profile(
            sender_id="senderId",
            about="Succulent specialists!",
            address="address",
            description="We specialize in providing high-quality succulents.",
            email="contact@example.com",
            vertical="RETAIL",
            websites=["https://www.example.com"],
        )
        assert_matches_type(SenderUpdateProfileResponse, sender, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_update_profile(self, client: Zavudev) -> None:
        response = client.senders.with_raw_response.update_profile(
            sender_id="senderId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        sender = response.parse()
        assert_matches_type(SenderUpdateProfileResponse, sender, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_update_profile(self, client: Zavudev) -> None:
        with client.senders.with_streaming_response.update_profile(
            sender_id="senderId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            sender = response.parse()
            assert_matches_type(SenderUpdateProfileResponse, sender, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_update_profile(self, client: Zavudev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `sender_id` but received ''"):
            client.senders.with_raw_response.update_profile(
                sender_id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_upload_profile_picture(self, client: Zavudev) -> None:
        sender = client.senders.upload_profile_picture(
            sender_id="senderId",
            image_url="https://example.com/profile.jpg",
            mime_type="image/jpeg",
        )
        assert_matches_type(SenderUploadProfilePictureResponse, sender, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_upload_profile_picture(self, client: Zavudev) -> None:
        response = client.senders.with_raw_response.upload_profile_picture(
            sender_id="senderId",
            image_url="https://example.com/profile.jpg",
            mime_type="image/jpeg",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        sender = response.parse()
        assert_matches_type(SenderUploadProfilePictureResponse, sender, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_upload_profile_picture(self, client: Zavudev) -> None:
        with client.senders.with_streaming_response.upload_profile_picture(
            sender_id="senderId",
            image_url="https://example.com/profile.jpg",
            mime_type="image/jpeg",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            sender = response.parse()
            assert_matches_type(SenderUploadProfilePictureResponse, sender, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_upload_profile_picture(self, client: Zavudev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `sender_id` but received ''"):
            client.senders.with_raw_response.upload_profile_picture(
                sender_id="",
                image_url="https://example.com/profile.jpg",
                mime_type="image/jpeg",
            )


class TestAsyncSenders:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncZavudev) -> None:
        sender = await async_client.senders.create(
            name="name",
            phone_number="phoneNumber",
        )
        assert_matches_type(Sender, sender, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncZavudev) -> None:
        sender = await async_client.senders.create(
            name="name",
            phone_number="phoneNumber",
            set_as_default=True,
            webhook_events=["message.queued"],
            webhook_url="https://example.com",
        )
        assert_matches_type(Sender, sender, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncZavudev) -> None:
        response = await async_client.senders.with_raw_response.create(
            name="name",
            phone_number="phoneNumber",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        sender = await response.parse()
        assert_matches_type(Sender, sender, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncZavudev) -> None:
        async with async_client.senders.with_streaming_response.create(
            name="name",
            phone_number="phoneNumber",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            sender = await response.parse()
            assert_matches_type(Sender, sender, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncZavudev) -> None:
        sender = await async_client.senders.retrieve(
            "senderId",
        )
        assert_matches_type(Sender, sender, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncZavudev) -> None:
        response = await async_client.senders.with_raw_response.retrieve(
            "senderId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        sender = await response.parse()
        assert_matches_type(Sender, sender, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncZavudev) -> None:
        async with async_client.senders.with_streaming_response.retrieve(
            "senderId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            sender = await response.parse()
            assert_matches_type(Sender, sender, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncZavudev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `sender_id` but received ''"):
            await async_client.senders.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update(self, async_client: AsyncZavudev) -> None:
        sender = await async_client.senders.update(
            sender_id="senderId",
        )
        assert_matches_type(Sender, sender, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncZavudev) -> None:
        sender = await async_client.senders.update(
            sender_id="senderId",
            email_receiving_enabled=True,
            name="name",
            set_as_default=True,
            webhook_active=True,
            webhook_events=["message.queued"],
            webhook_url="https://example.com",
        )
        assert_matches_type(Sender, sender, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_update(self, async_client: AsyncZavudev) -> None:
        response = await async_client.senders.with_raw_response.update(
            sender_id="senderId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        sender = await response.parse()
        assert_matches_type(Sender, sender, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncZavudev) -> None:
        async with async_client.senders.with_streaming_response.update(
            sender_id="senderId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            sender = await response.parse()
            assert_matches_type(Sender, sender, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_update(self, async_client: AsyncZavudev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `sender_id` but received ''"):
            await async_client.senders.with_raw_response.update(
                sender_id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncZavudev) -> None:
        sender = await async_client.senders.list()
        assert_matches_type(AsyncCursor[Sender], sender, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncZavudev) -> None:
        sender = await async_client.senders.list(
            cursor="cursor",
            limit=100,
        )
        assert_matches_type(AsyncCursor[Sender], sender, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncZavudev) -> None:
        response = await async_client.senders.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        sender = await response.parse()
        assert_matches_type(AsyncCursor[Sender], sender, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncZavudev) -> None:
        async with async_client.senders.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            sender = await response.parse()
            assert_matches_type(AsyncCursor[Sender], sender, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_delete(self, async_client: AsyncZavudev) -> None:
        sender = await async_client.senders.delete(
            "senderId",
        )
        assert sender is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncZavudev) -> None:
        response = await async_client.senders.with_raw_response.delete(
            "senderId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        sender = await response.parse()
        assert sender is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncZavudev) -> None:
        async with async_client.senders.with_streaming_response.delete(
            "senderId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            sender = await response.parse()
            assert sender is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_delete(self, async_client: AsyncZavudev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `sender_id` but received ''"):
            await async_client.senders.with_raw_response.delete(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_get_profile(self, async_client: AsyncZavudev) -> None:
        sender = await async_client.senders.get_profile(
            "senderId",
        )
        assert_matches_type(WhatsappBusinessProfileResponse, sender, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_get_profile(self, async_client: AsyncZavudev) -> None:
        response = await async_client.senders.with_raw_response.get_profile(
            "senderId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        sender = await response.parse()
        assert_matches_type(WhatsappBusinessProfileResponse, sender, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_get_profile(self, async_client: AsyncZavudev) -> None:
        async with async_client.senders.with_streaming_response.get_profile(
            "senderId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            sender = await response.parse()
            assert_matches_type(WhatsappBusinessProfileResponse, sender, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_get_profile(self, async_client: AsyncZavudev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `sender_id` but received ''"):
            await async_client.senders.with_raw_response.get_profile(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_regenerate_webhook_secret(self, async_client: AsyncZavudev) -> None:
        sender = await async_client.senders.regenerate_webhook_secret(
            "senderId",
        )
        assert_matches_type(WebhookSecretResponse, sender, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_regenerate_webhook_secret(self, async_client: AsyncZavudev) -> None:
        response = await async_client.senders.with_raw_response.regenerate_webhook_secret(
            "senderId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        sender = await response.parse()
        assert_matches_type(WebhookSecretResponse, sender, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_regenerate_webhook_secret(self, async_client: AsyncZavudev) -> None:
        async with async_client.senders.with_streaming_response.regenerate_webhook_secret(
            "senderId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            sender = await response.parse()
            assert_matches_type(WebhookSecretResponse, sender, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_regenerate_webhook_secret(self, async_client: AsyncZavudev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `sender_id` but received ''"):
            await async_client.senders.with_raw_response.regenerate_webhook_secret(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update_profile(self, async_client: AsyncZavudev) -> None:
        sender = await async_client.senders.update_profile(
            sender_id="senderId",
        )
        assert_matches_type(SenderUpdateProfileResponse, sender, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update_profile_with_all_params(self, async_client: AsyncZavudev) -> None:
        sender = await async_client.senders.update_profile(
            sender_id="senderId",
            about="Succulent specialists!",
            address="address",
            description="We specialize in providing high-quality succulents.",
            email="contact@example.com",
            vertical="RETAIL",
            websites=["https://www.example.com"],
        )
        assert_matches_type(SenderUpdateProfileResponse, sender, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_update_profile(self, async_client: AsyncZavudev) -> None:
        response = await async_client.senders.with_raw_response.update_profile(
            sender_id="senderId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        sender = await response.parse()
        assert_matches_type(SenderUpdateProfileResponse, sender, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_update_profile(self, async_client: AsyncZavudev) -> None:
        async with async_client.senders.with_streaming_response.update_profile(
            sender_id="senderId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            sender = await response.parse()
            assert_matches_type(SenderUpdateProfileResponse, sender, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_update_profile(self, async_client: AsyncZavudev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `sender_id` but received ''"):
            await async_client.senders.with_raw_response.update_profile(
                sender_id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_upload_profile_picture(self, async_client: AsyncZavudev) -> None:
        sender = await async_client.senders.upload_profile_picture(
            sender_id="senderId",
            image_url="https://example.com/profile.jpg",
            mime_type="image/jpeg",
        )
        assert_matches_type(SenderUploadProfilePictureResponse, sender, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_upload_profile_picture(self, async_client: AsyncZavudev) -> None:
        response = await async_client.senders.with_raw_response.upload_profile_picture(
            sender_id="senderId",
            image_url="https://example.com/profile.jpg",
            mime_type="image/jpeg",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        sender = await response.parse()
        assert_matches_type(SenderUploadProfilePictureResponse, sender, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_upload_profile_picture(self, async_client: AsyncZavudev) -> None:
        async with async_client.senders.with_streaming_response.upload_profile_picture(
            sender_id="senderId",
            image_url="https://example.com/profile.jpg",
            mime_type="image/jpeg",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            sender = await response.parse()
            assert_matches_type(SenderUploadProfilePictureResponse, sender, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_upload_profile_picture(self, async_client: AsyncZavudev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `sender_id` but received ''"):
            await async_client.senders.with_raw_response.upload_profile_picture(
                sender_id="",
                image_url="https://example.com/profile.jpg",
                mime_type="image/jpeg",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["AgentStats"]


class AgentStats(BaseModel):
    error_count: int = FieldInfo(alias="errorCount")

    success_count: int = FieldInfo(alias="successCount")

    total_cost: float = FieldInfo(alias="totalCost")
    """Total cost in USD."""

    total_invocations: int = FieldInfo(alias="totalInvocations")

    total_tokens_used: int = FieldInfo(alias="totalTokensUsed")

    avg_latency_ms: Optional[float] = FieldInfo(alias="avgLatencyMs", default=None)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Optional
from datetime import datetime

from pydantic import Field as FieldInfo

from .._models import BaseModel
from .broadcast_status import BroadcastStatus
from .broadcast_channel import BroadcastChannel
from .broadcast_content import BroadcastContent
from .broadcast_message_type import BroadcastMessageType

__all__ = ["Broadcast", "ReviewResult"]


class ReviewResult(BaseModel):
    """AI content review result."""

    categories: Optional[List[str]] = None
    """Policy categories violated, if any."""

    flagged_content: Optional[List[str]] = FieldInfo(alias="flaggedContent", default=None)
    """Problematic text fragments, if any."""

    reasoning: Optional[str] = None
    """Explanation of the review decision."""

    reviewed_at: Optional[datetime] = FieldInfo(alias="reviewedAt", default=None)

    score: Optional[float] = None
    """Content safety score from 0.0 to 1.0, where 1.0 is completely safe."""


class Broadcast(BaseModel):
    id: str

    channel: BroadcastChannel
    """Broadcast delivery channel. Use 'smart' for per-contact intelligent routing."""

    created_at: datetime = FieldInfo(alias="createdAt")

    message_type: BroadcastMessageType = FieldInfo(alias="messageType")
    """Type of message for broadcast."""

    name: str

    status: BroadcastStatus
    """Current status of the broadcast."""

    total_contacts: int = FieldInfo(alias="totalContacts")
    """Total number of contacts in the broadcast."""

    actual_cost: Optional[float] = FieldInfo(alias="actualCost", default=None)
    """Actual cost so far in USD."""

    completed_at: Optional[datetime] = FieldInfo(alias="completedAt", default=None)

    content: Optional[BroadcastContent] = None
    """Content for non-text broadcast message types."""

    delivered_count: Optional[int] = FieldInfo(alias="deliveredCount", default=None)

    email_subject: Optional[str] = FieldInfo(alias="emailSubject", default=None)

    estimated_cost: Optional[float] = FieldInfo(alias="estimatedCost", default=None)
    """Estimated total cost in USD."""

    failed_count: Optional[int] = FieldInfo(alias="failedCount", default=None)

    metadata: Optional[Dict[str, str]] = None

    pending_count: Optional[int] = FieldInfo(alias="pendingCount", default=None)

    reserved_amount: Optional[float] = FieldInfo(alias="reservedAmount", default=None)
    """Amount reserved from balance in USD."""

    review_attempts: Optional[int] = FieldInfo(alias="reviewAttempts", default=None)
    """Number of review attempts (max 3)."""

    review_result: Optional[ReviewResult] = FieldInfo(alias="reviewResult", default=None)
    """AI content review result."""

    scheduled_at: Optional[datetime] = FieldInfo(alias="scheduledAt", default=None)

    sender_id: Optional[str] = FieldInfo(alias="senderId", default=None)

    sending_count: Optional[int] = FieldInfo(alias="sendingCount", default=None)

    started_at: Optional[datetime] = FieldInfo(alias="startedAt", default=None)

    text: Optional[str] = None

    updated_at: Optional[datetime] = FieldInfo(alias="updatedAt", default=None)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ..._models import BaseModel
from .agent.agent import Agent

__all__ = ["AgentResponse"]


class AgentResponse(BaseModel):
    agent: Agent
    """AI Agent configuration for a sender."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal, TypeAlias

__all__ = ["BroadcastStatus"]

BroadcastStatus: TypeAlias = Literal[
    "draft",
    "pending_review",
    "approved",
    "rejected",
    "escalated",
    "rejected_final",
    "scheduled",
    "sending",
    "paused",
    "completed",
    "cancelled",
    "failed",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from zavudev import Zavudev, AsyncZavudev
from tests.utils import assert_matches_type
from zavudev.types.senders import AgentStats, AgentResponse

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestAgent:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create(self, client: Zavudev) -> None:
        agent = client.senders.agent.create(
            sender_id="senderId",
            model="gpt-4o-mini",
            name="Customer Support",
            provider="openai",
            system_prompt="You are a helpful customer support agent. Be friendly and concise.",
        )
        assert_matches_type(AgentResponse, agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create_with_all_params(self, client: Zavudev) -> None:
        agent = client.senders.agent.create(
            sender_id="senderId",
            model="gpt-4o-mini",
            name="Customer Support",
            provider="openai",
            system_prompt="You are a helpful customer support agent. Be friendly and concise.",
            api_key="sk-...",
            context_window_messages=1,
            include_contact_metadata=True,
            max_tokens=1,
            temperature=0,
            trigger_on_channels=["string"],
            trigger_on_message_types=["string"],
        )
        assert_matches_type(AgentResponse, agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Zavudev) -> None:
        response = client.senders.agent.with_raw_response.create(
            sender_id="senderId",
            model="gpt-4o-mini",
            name="Customer Support",
            provider="openai",
            system_prompt="You are a helpful customer support agent. Be friendly and concise.",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        agent = response.parse()
        assert_matches_type(AgentResponse, agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Zavudev) -> None:
        with client.senders.agent.with_streaming_response.create(
            sender_id="senderId",
            model="gpt-4o-mini",
            name="Customer Support",
            provider="openai",
            system_prompt="You are a helpful customer support agent. Be friendly and concise.",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            agent = response.parse()
            assert_matches_type(AgentResponse, agent, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_create(self, client: Zavudev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `sender_id` but received ''"):
            client.senders.agent.with_raw_response.create(
                sender_id="",
                model="gpt-4o-mini",
                name="Customer Support",
                provider="openai",
                system_prompt="You are a helpful customer support agent. Be friendly and concise.",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Zavudev) -> None:
        agent = client.senders.agent.retrieve(
            "senderId",
        )
        assert_matches_type(AgentResponse, agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Zavudev) -> None:
        response = client.senders.agent.with_raw_response.retrieve(
            "senderId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        agent = response.parse()
        assert_matches_type(AgentResponse, agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Zavudev) -> None:
        with client.senders.agent.with_streaming_response.retrieve(
            "senderId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            agent = response.parse()
            assert_matches_type(AgentResponse, agent, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Zavudev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `sender_id` but received ''"):
            client.senders.agent.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update(self, client: Zavudev) -> None:
        agent = client.senders.agent.update(
            sender_id="senderId",
        )
        assert_matches_type(AgentResponse, agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update_with_all_params(self, client: Zavudev) -> None:
        agent = client.senders.agent.update(
            sender_id="senderId",
            api_key="apiKey",
            context_window_messages=1,
            enabled=True,
            include_contact_metadata=True,
            max_tokens=1,
            model="model",
            name="name",
            provider="openai",
            system_prompt="systemPrompt",
            temperature=0,
            trigger_on_channels=["string"],
            trigger_on_message_types=["string"],
        )
        assert_matches_type(AgentResponse, agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_update(self, client: Zavudev) -> None:
        response = client.senders.agent.with_raw_response.update(
            sender_id="senderId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        agent = response.parse()
        assert_matches_type(AgentResponse, agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_update(self, client: Zavudev) -> None:
        with client.senders.agent.with_streaming_response.update(
            sender_id="senderId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            agent = response.parse()
            assert_matches_type(AgentResponse, agent, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_update(self, client: Zavudev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `sender_id` but received ''"):
            client.senders.agent.with_raw_response.update(
                sender_id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_delete(self, client: Zavudev) -> None:
        agent = client.senders.agent.delete(
            "senderId",
        )
        assert agent is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_delete(self, client: Zavudev) -> None:
        response = client.senders.agent.with_raw_response.delete(
            "senderId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        agent = response.parse()
        assert agent is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_delete(self, client: Zavudev) -> None:
        with client.senders.agent.with_streaming_response.delete(
            "senderId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            agent = response.parse()
            assert agent is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_delete(self, client: Zavudev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `sender_id` but received ''"):
            client.senders.agent.with_raw_response.delete(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_stats(self, client: Zavudev) -> None:
        agent = client.senders.agent.stats(
            "senderId",
        )
        assert_matches_type(AgentStats, agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_stats(self, client: Zavudev) -> None:
        response = client.senders.agent.with_raw_response.stats(
            "senderId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        agent = response.parse()
        assert_matches_type(AgentStats, agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_stats(self, client: Zavudev) -> None:
        with client.senders.agent.with_streaming_response.stats(
            "senderId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            agent = response.parse()
            assert_matches_type(AgentStats, agent, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_stats(self, client: Zavudev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `sender_id` but received ''"):
            client.senders.agent.with_raw_response.stats(
                "",
            )


class TestAsyncAgent:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncZavudev) -> None:
        agent = await async_client.senders.agent.create(
            sender_id="senderId",
            model="gpt-4o-mini",
            name="Customer Support",
            provider="openai",
            system_prompt="You are a helpful customer support agent. Be friendly and concise.",
        )
        assert_matches_type(AgentResponse, agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncZavudev) -> None:
        agent = await async_client.senders.agent.create(
            sender_id="senderId",
            model="gpt-4o-mini",
            name="Customer Support",
            provider="openai",
            system_prompt="You are a helpful customer support agent. Be friendly and concise.",
            api_key="sk-...",
            context_window_messages=1,
            include_contact_metadata=True,
            max_tokens=1,
            temperature=0,
            trigger_on_channels=["string"],
            trigger_on_message_types=["string"],
        )
        assert_matches_type(AgentResponse, agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncZavudev) -> None:
        response = await async_client.senders.agent.with_raw_response.create(
            sender_id="senderId",
            model="gpt-4o-mini",
            name="Customer Support",
            provider="openai",
            system_prompt="You are a helpful customer support agent. Be friendly and concise.",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        agent = await response.parse()
        assert_matches_type(AgentResponse, agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncZavudev) -> None:
        async with async_client.senders.agent.with_streaming_response.create(
            sender_id="senderId",
            model="gpt-4o-mini",
            name="Customer Support",
            provider="openai",
            system_prompt="You are a helpful customer support agent. Be friendly and concise.",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            agent = await response.parse()
            assert_matches_type(AgentResponse, agent, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_create(self, async_client: AsyncZavudev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `sender_id` but received ''"):
            await async_client.senders.agent.with_raw_response.create(
                sender_id="",
                model="gpt-4o-mini",
                name="Customer Support",
                provider="openai",
                system_prompt="You are a helpful customer support agent. Be friendly and concise.",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncZavudev) -> None:
        agent = await async_client.senders.agent.retrieve(
            "senderId",
        )
        assert_matches_type(AgentResponse, agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncZavudev) -> None:
        response = await async_client.senders.agent.with_raw_response.retrieve(
            "senderId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        agent = await response.parse()
        assert_matches_type(AgentResponse, agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncZavudev) -> None:
        async with async_client.senders.agent.with_streaming_response.retrieve(
            "senderId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            agent = await response.parse()
            assert_matches_type(AgentResponse, agent, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncZavudev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `sender_id` but received ''"):
            await async_client.senders.agent.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update(self, async_client: AsyncZavudev) -> None:
        agent = await async_client.senders.agent.update(
            sender_id="senderId",
        )
        assert_matches_type(AgentResponse, agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncZavudev) -> None:
        agent = await async_client.senders.agent.update(
            sender_id="senderId",
            api_key="apiKey",
            context_window_messages=1,
            enabled=True,
            include_contact_metadata=True,
            max_tokens=1,
            model="model",
            name="name",
            provider="openai",
            system_prompt="systemPrompt",
            temperature=0,
            trigger_on_channels=["string"],
            trigger_on_message_types=["string"],
        )
        assert_matches_type(AgentResponse, agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_update(self, async_client: AsyncZavudev) -> None:
        response = await async_client.senders.agent.with_raw_response.update(
            sender_id="senderId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        agent = await response.parse()
        assert_matches_type(AgentResponse, agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncZavudev) -> None:
        async with async_client.senders.agent.with_streaming_response.update(
            sender_id="senderId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            agent = await response.parse()
            assert_matches_type(AgentResponse, agent, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_update(self, async_client: AsyncZavudev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `sender_id` but received ''"):
            await async_client.senders.agent.with_raw_response.update(
                sender_id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_delete(self, async_client: AsyncZavudev) -> None:
        agent = await async_client.senders.agent.delete(
            "senderId",
        )
        assert agent is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncZavudev) -> None:
        response = await async_client.senders.agent.with_raw_response.delete(
            "senderId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        agent = await response.parse()
        assert agent is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncZavudev) -> None:
        async with async_client.senders.agent.with_streaming_response.delete(
            "senderId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            agent = await response.parse()
            assert agent is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_delete(self, async_client: AsyncZavudev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `sender_id` but received ''"):
            await async_client.senders.agent.with_raw_response.delete(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_stats(self, async_client: AsyncZavudev) -> None:
        agent = await async_client.senders.agent.stats(
            "senderId",
        )
        assert_matches_type(AgentStats, agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_stats(self, async_client: AsyncZavudev) -> None:
        response = await async_client.senders.agent.with_raw_response.stats(
            "senderId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        agent = await response.parse()
        assert_matches_type(AgentStats, agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_stats(self, async_client: AsyncZavudev) -> None:
        async with async_client.senders.agent.with_streaming_response.stats(
            "senderId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            agent = await response.parse()
            assert_matches_type(AgentStats, agent, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_stats(self, async_client: AsyncZavudev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `sender_id` but received ''"):
            await async_client.senders.agent.with_raw_response.stats(
                "",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import datetime
from typing_extensions import Required, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["BroadcastRescheduleParams"]


class BroadcastRescheduleParams(TypedDict, total=False):
    scheduled_at: Required[Annotated[Union[str, datetime], PropertyInfo(alias="scheduledAt", format="iso8601")]]
    """New scheduled time for the broadcast."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .agent import Agent as Agent
from .agent_stats import AgentStats as AgentStats
from .agent_provider import AgentProvider as AgentProvider
from .agent_response import AgentResponse as AgentResponse
from .agent_execution import AgentExecution as AgentExecution
from .agent_create_params import AgentCreateParams as AgentCreateParams
from .agent_update_params import AgentUpdateParams as AgentUpdateParams
from .agent_execution_status import AgentExecutionStatus as AgentExecutionStatus



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Literal, Required, Annotated, TypedDict

from .._types import SequenceNotStr
from .._utils import PropertyInfo
from .whatsapp_category import WhatsappCategory

__all__ = ["TemplateCreateParams", "Button"]


class TemplateCreateParams(TypedDict, total=False):
    body: Required[str]

    language: Required[str]

    name: Required[str]

    add_security_recommendation: Annotated[bool, PropertyInfo(alias="addSecurityRecommendation")]
    """Add 'Do not share this code' disclaimer. Only for AUTHENTICATION templates."""

    buttons: Iterable[Button]
    """Template buttons (max 3)."""

    code_expiration_minutes: Annotated[int, PropertyInfo(alias="codeExpirationMinutes")]
    """Code expiration time in minutes. Only for AUTHENTICATION templates."""

    variables: SequenceNotStr[str]

    whatsapp_category: Annotated[WhatsappCategory, PropertyInfo(alias="whatsappCategory")]
    """WhatsApp template category."""


class Button(TypedDict, total=False):
    text: Required[str]

    type: Required[Literal["quick_reply", "url", "phone", "otp"]]

    otp_type: Annotated[Literal["COPY_CODE", "ONE_TAP"], PropertyInfo(alias="otpType")]
    """Required when type is 'otp'.

    COPY_CODE shows copy button, ONE_TAP enables Android autofill.
    """

    package_name: Annotated[str, PropertyInfo(alias="packageName")]
    """Android package name. Required for ONE_TAP buttons."""

    phone_number: Annotated[str, PropertyInfo(alias="phoneNumber")]

    signature_hash: Annotated[str, PropertyInfo(alias="signatureHash")]
    """Android app signature hash. Required for ONE_TAP buttons."""

    url: str



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal, TypeAlias

__all__ = ["PhoneNumberStatus"]

PhoneNumberStatus: TypeAlias = Literal["active", "suspended", "pending"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime

from pydantic import Field as FieldInfo

from ...._models import BaseModel

__all__ = ["AgentKnowledgeBase"]


class AgentKnowledgeBase(BaseModel):
    id: str

    agent_id: str = FieldInfo(alias="agentId")

    created_at: datetime = FieldInfo(alias="createdAt")

    document_count: int = FieldInfo(alias="documentCount")

    name: str

    total_chunks: int = FieldInfo(alias="totalChunks")

    updated_at: datetime = FieldInfo(alias="updatedAt")

    description: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Optional
from typing_extensions import Annotated, TypedDict

from .._utils import PropertyInfo
from .webhook_event import WebhookEvent

__all__ = ["SenderUpdateParams"]


class SenderUpdateParams(TypedDict, total=False):
    email_receiving_enabled: Annotated[bool, PropertyInfo(alias="emailReceivingEnabled")]
    """Enable or disable inbound email receiving for this sender."""

    name: str

    set_as_default: Annotated[bool, PropertyInfo(alias="setAsDefault")]

    webhook_active: Annotated[bool, PropertyInfo(alias="webhookActive")]
    """Whether the webhook is active."""

    webhook_events: Annotated[List[WebhookEvent], PropertyInfo(alias="webhookEvents")]
    """Events to subscribe to."""

    webhook_url: Annotated[Optional[str], PropertyInfo(alias="webhookUrl")]
    """HTTPS URL for webhook events. Set to null to remove webhook."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from pydantic import Field as FieldInfo

from .._models import BaseModel
from .phone_number_pricing import PhoneNumberPricing
from .phone_number_capabilities import PhoneNumberCapabilities

__all__ = ["AvailablePhoneNumber"]


class AvailablePhoneNumber(BaseModel):
    capabilities: PhoneNumberCapabilities

    phone_number: str = FieldInfo(alias="phoneNumber")

    pricing: PhoneNumberPricing

    friendly_name: Optional[str] = FieldInfo(alias="friendlyName", default=None)

    locality: Optional[str] = None

    region: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Optional
from typing_extensions import Literal, Required, Annotated, TypedDict

from ...._types import SequenceNotStr
from ...._utils import PropertyInfo

__all__ = ["ToolUpdateParams", "Parameters", "ParametersProperties"]


class ToolUpdateParams(TypedDict, total=False):
    sender_id: Required[Annotated[str, PropertyInfo(alias="senderId")]]

    description: str

    enabled: bool

    name: str

    parameters: Parameters

    webhook_secret: Annotated[Optional[str], PropertyInfo(alias="webhookSecret")]

    webhook_url: Annotated[str, PropertyInfo(alias="webhookUrl")]


class ParametersProperties(TypedDict, total=False):
    description: str

    type: str


class Parameters(TypedDict, total=False):
    properties: Required[Dict[str, ParametersProperties]]

    required: Required[SequenceNotStr[str]]

    type: Required[Literal["object"]]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .sender import Sender as Sender
from .address import Address as Address
from .channel import Channel as Channel
from .contact import Contact as Contact
from .message import Message as Message
from .template import Template as Template
from .broadcast import Broadcast as Broadcast
from .line_type import LineType as LineType
from .requirement import Requirement as Requirement
from .message_type import MessageType as MessageType
from .webhook_event import WebhookEvent as WebhookEvent
from .address_status import AddressStatus as AddressStatus
from .message_status import MessageStatus as MessageStatus
from .sender_webhook import SenderWebhook as SenderWebhook
from .message_content import MessageContent as MessageContent
from .broadcast_status import BroadcastStatus as BroadcastStatus
from .message_response import MessageResponse as MessageResponse
from .requirement_type import RequirementType as RequirementType
from .broadcast_channel import BroadcastChannel as BroadcastChannel
from .broadcast_contact import BroadcastContact as BroadcastContact
from .broadcast_content import BroadcastContent as BroadcastContent
from .phone_number_type import PhoneNumberType as PhoneNumberType
from .whatsapp_category import WhatsappCategory as WhatsappCategory
from .broadcast_progress import BroadcastProgress as BroadcastProgress
from .owned_phone_number import OwnedPhoneNumber as OwnedPhoneNumber
from .sender_list_params import SenderListParams as SenderListParams
from .address_list_params import AddressListParams as AddressListParams
from .contact_list_params import ContactListParams as ContactListParams
from .message_list_params import MessageListParams as MessageListParams
from .message_send_params import MessageSendParams as MessageSendParams
from .phone_number_status import PhoneNumberStatus as PhoneNumberStatus
from .regulatory_document import RegulatoryDocument as RegulatoryDocument
from .message_react_params import MessageReactParams as MessageReactParams
from .phone_number_pricing import PhoneNumberPricing as PhoneNumberPricing
from .sender_create_params import SenderCreateParams as SenderCreateParams
from .sender_update_params import SenderUpdateParams as SenderUpdateParams
from .template_list_params import TemplateListParams as TemplateListParams
from .address_create_params import AddressCreateParams as AddressCreateParams
from .broadcast_list_params import BroadcastListParams as BroadcastListParams
from .broadcast_send_params import BroadcastSendParams as BroadcastSendParams
from .contact_update_params import ContactUpdateParams as ContactUpdateParams
from .message_content_param import MessageContentParam as MessageContentParam
from .available_phone_number import AvailablePhoneNumber as AvailablePhoneNumber
from .broadcast_message_type import BroadcastMessageType as BroadcastMessageType
from .requirement_field_type import RequirementFieldType as RequirementFieldType
from .template_create_params import TemplateCreateParams as TemplateCreateParams
from .template_submit_params import TemplateSubmitParams as TemplateSubmitParams
from .address_create_response import AddressCreateResponse as AddressCreateResponse
from .broadcast_content_param import BroadcastContentParam as BroadcastContentParam
from .broadcast_create_params import BroadcastCreateParams as BroadcastCreateParams
from .broadcast_send_response import BroadcastSendResponse as BroadcastSendResponse
from .broadcast_update_params import BroadcastUpdateParams as BroadcastUpdateParams
from .webhook_secret_response import WebhookSecretResponse as WebhookSecretResponse
from .broadcast_contact_status import BroadcastContactStatus as BroadcastContactStatus
from .phone_number_list_params import PhoneNumberListParams as PhoneNumberListParams
from .address_retrieve_response import AddressRetrieveResponse as AddressRetrieveResponse
from .broadcast_cancel_response import BroadcastCancelResponse as BroadcastCancelResponse
from .broadcast_create_response import BroadcastCreateResponse as BroadcastCreateResponse
from .broadcast_update_response import BroadcastUpdateResponse as BroadcastUpdateResponse
from .phone_number_capabilities import PhoneNumberCapabilities as PhoneNumberCapabilities
from .whatsapp_business_profile import WhatsappBusinessProfile as WhatsappBusinessProfile
from .owned_phone_number_pricing import OwnedPhoneNumberPricing as OwnedPhoneNumberPricing
from .phone_number_update_params import PhoneNumberUpdateParams as PhoneNumberUpdateParams
from .broadcast_reschedule_params import BroadcastRescheduleParams as BroadcastRescheduleParams
from .broadcast_retrieve_response import BroadcastRetrieveResponse as BroadcastRetrieveResponse
from .phone_number_purchase_params import PhoneNumberPurchaseParams as PhoneNumberPurchaseParams
from .phone_number_update_response import PhoneNumberUpdateResponse as PhoneNumberUpdateResponse
from .sender_update_profile_params import SenderUpdateProfileParams as SenderUpdateProfileParams
from .broadcast_reschedule_response import BroadcastRescheduleResponse as BroadcastRescheduleResponse
from .phone_number_purchase_response import PhoneNumberPurchaseResponse as PhoneNumberPurchaseResponse
from .phone_number_retrieve_response import PhoneNumberRetrieveResponse as PhoneNumberRetrieveResponse
from .sender_update_profile_response import SenderUpdateProfileResponse as SenderUpdateProfileResponse
from .regulatory_document_list_params import RegulatoryDocumentListParams as RegulatoryDocumentListParams
from .requirement_acceptance_criteria import RequirementAcceptanceCriteria as RequirementAcceptanceCriteria
from .introspect_validate_phone_params import IntrospectValidatePhoneParams as IntrospectValidatePhoneParams
from .phone_number_requirements_params import PhoneNumberRequirementsParams as PhoneNumberRequirementsParams
from .regulatory_document_create_params import RegulatoryDocumentCreateParams as RegulatoryDocumentCreateParams
from .introspect_validate_phone_response import IntrospectValidatePhoneResponse as IntrospectValidatePhoneResponse
from .phone_number_requirements_response import PhoneNumberRequirementsResponse as PhoneNumberRequirementsResponse
from .whatsapp_business_profile_response import WhatsappBusinessProfileResponse as WhatsappBusinessProfileResponse
from .whatsapp_business_profile_vertical import WhatsappBusinessProfileVertical as WhatsappBusinessProfileVertical
from .regulatory_document_create_response import RegulatoryDocumentCreateResponse as RegulatoryDocumentCreateResponse
from .phone_number_search_available_params import PhoneNumberSearchAvailableParams as PhoneNumberSearchAvailableParams
from .sender_upload_profile_picture_params import SenderUploadProfilePictureParams as SenderUploadProfilePictureParams
from .regulatory_document_retrieve_response import (
    RegulatoryDocumentRetrieveResponse as RegulatoryDocumentRetrieveResponse,
)
from .phone_number_search_available_response import (
    PhoneNumberSearchAvailableResponse as PhoneNumberSearchAvailableResponse,
)
from .sender_upload_profile_picture_response import (
    SenderUploadProfilePictureResponse as SenderUploadProfilePictureResponse,
)
from .regulatory_document_upload_url_response import (
    RegulatoryDocumentUploadURLResponse as RegulatoryDocumentUploadURLResponse,
)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["PhoneNumberPurchaseParams"]


class PhoneNumberPurchaseParams(TypedDict, total=False):
    phone_number: Required[Annotated[str, PropertyInfo(alias="phoneNumber")]]
    """Phone number in E.164 format."""

    name: str
    """Optional custom name for the phone number."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal, TypeAlias

__all__ = ["WebhookEvent"]

WebhookEvent: TypeAlias = Literal[
    "message.queued",
    "message.sent",
    "message.delivered",
    "message.failed",
    "message.inbound",
    "message.unsupported",
    "conversation.new",
    "template.status_changed",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel

__all__ = ["PhoneNumberCapabilities"]


class PhoneNumberCapabilities(BaseModel):
    mms: Optional[bool] = None

    sms: Optional[bool] = None

    voice: Optional[bool] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime

from pydantic import Field as FieldInfo

from .._models import BaseModel
from .phone_number_status import PhoneNumberStatus
from .owned_phone_number_pricing import OwnedPhoneNumberPricing

__all__ = ["OwnedPhoneNumber"]


class OwnedPhoneNumber(BaseModel):
    id: str

    capabilities: List[str]

    created_at: datetime = FieldInfo(alias="createdAt")

    phone_number: str = FieldInfo(alias="phoneNumber")

    pricing: OwnedPhoneNumberPricing

    status: PhoneNumberStatus

    name: Optional[str] = None
    """Optional custom name for the phone number."""

    next_renewal_date: Optional[datetime] = FieldInfo(alias="nextRenewalDate", default=None)

    sender_id: Optional[str] = FieldInfo(alias="senderId", default=None)
    """Sender ID if the phone number is assigned to a sender."""

    updated_at: Optional[datetime] = FieldInfo(alias="updatedAt", default=None)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Required, Annotated, TypedDict

from .._utils import PropertyInfo
from .webhook_event import WebhookEvent

__all__ = ["SenderCreateParams"]


class SenderCreateParams(TypedDict, total=False):
    name: Required[str]

    phone_number: Required[Annotated[str, PropertyInfo(alias="phoneNumber")]]

    set_as_default: Annotated[bool, PropertyInfo(alias="setAsDefault")]

    webhook_events: Annotated[List[WebhookEvent], PropertyInfo(alias="webhookEvents")]
    """Events to subscribe to."""

    webhook_url: Annotated[str, PropertyInfo(alias="webhookUrl")]
    """HTTPS URL for webhook events."""
